<?php
/**
 * Joomla! Yireo Library
 *
 * @author Yireo (https://www.yireo.com/)
 * @package YireoLib
 * @copyright Copyright 2011
 * @license GNU Public License
 * @link https://www.yireo.com/
 * @version 0.4.2
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

// Require the parent view
require_once dirname(dirname(__FILE__)).DS.'view.php';

/**
 * List View class
 *
 * @package Yireo
 */
class YireoViewList extends YireoView
{
    /*
     * Flag to determine whether to load edit/copy/new buttons
     */
    protected $loadToolbarEdit = true;

    /*
     * Flag to determine whether to load delete buttons
     */
    protected $loadToolbarDelete = true;

    /*
     * Main constructor method
     *
     * @access public
     * @subpackage Yireo
     * @param null
     * @return null
     */
    public function __construct()
    {
        // Template-paths
        $this->templatePaths[] = dirname(__FILE__).DS.'list';

        // Do not load the toolbar automatically
        $this->loadToolbar = false;

        // Call the parent constructor
        return parent::__construct();
    }

    /*
     * Main display method
     *
     * @access public
     * @subpackage Yireo
     * @param string $tpl
     * @return null
     */
    public function display($tpl = null)
    {
        // Automatically fetch items, total and pagination - and assign them to the template
        $this->fetchItems();

        // Load model and table
        $model = $this->getModel();
        $primaryKey = $model->getPrimaryKey();
        $table = $model->getTable();

        // Parse the items a bit more
        if(!empty($this->items)) {
            foreach($this->items as $index => $item) {

                // Determine the primary key
                $item->id = $item->$primaryKey;

                // Set the various links
                $item->edit_link = JRoute::_('index.php?option='.$this->_option.'&view='.$this->_view.'&task=edit&cid[]='. $item->id);

                // Re-insert the item
                $this->items[$index] = $item;
            }
        }

        // Initialize the toolbar
        if($table->getStateField() != '') {
            JToolBarHelper::publishList();
            JToolBarHelper::unpublishList();
        }

        // Add the delete-button
        if($this->loadToolbarDelete == true) {
            JToolBarHelper::deleteList();
        }

        // Load the toolbar edit-buttons
        if($this->loadToolbarEdit == true) {
            JToolBarHelper::editListX();
            JToolBarHelper::custom( 'copy', 'copy.png', 'copy.png', 'Copy', true, true);
            JToolBarHelper::addNewX();
        }

        // Insert extra field
        $fields = array();
        $fields['primary_field'] = $primaryKey;
        $fields['ordering_field'] = $model->getOrderByDefault();
        $fields['state_field'] = $table->getStateField();
        $this->assignRef('fields', $fields);

        // Add an extra search-option
        $search = '<div class="filter-search">';
        $search .= '<label class="filter-search-lbl" for="search">'.JText::_( 'Filter').' </label>';
        $search .= '<input type="text" name="'.$this->lists['search_name'].'" id="search" value="'.$this->lists['search'].'" class="text_area" onchange="document.adminForm.submit();" />';
        $search .= '<button class="btn" onclick="this.form.submit();">'.JText::_( 'Go' ).'</button>';
        $search .= '<button class="btn" onclick="document.getElementById(\'search\').value=\'\';this.form.submit();">'.JText::_( 'Reset' ).'</button>';
        $search .= '</div>';
        $this->assignRef('search', $search);

        // Add extra hidden fields
        $hidden_fields = '<input type="hidden" name="option" value="'.$this->_option.'" />';
        $hidden_fields .= '<input type="hidden" name="view" value="'.$this->_view.'" />';
        $hidden_fields .= '<input type="hidden" name="task" value="" />';
        $hidden_fields .= '<input type="hidden" name="boxchecked" value="0" />';
        $hidden_fields .= '<input type="hidden" name="filter_order" value="'.$this->lists['order'].'" />';
        $hidden_fields .= '<input type="hidden" name="filter_order_Dir" value="" />';
        $hidden_fields .= JHTML::_( 'form.token' );
        $this->assignRef('hidden_fields', $hidden_fields);

        parent::display($tpl);
    }

    /*
     * Method to allow toggling a certain field
     *
     * @access public
     * @subpackage Yireo
     * @param string $name
     * @param string $value
     * @param boolean $ajax
     * @param int $id
     * @return null
     */
    public function toggle($name, $value, $ajax = false, $id = 0)
    {
        if($value == 1 || !empty($value)) {
            $img = 'images/tick.png';
        } else {
            $img = 'images/publish_x.png';
        }

        if($ajax == false) {
            return '<img src="'.JURI::base().$img.'" />';
        } else {
            $token = JUtility::getToken();
            $url = JRoute::_('index.php?option='.$this->_option.'&view='.$this->_view.'&task=toggle&id='.$id.'&name='.$name.'&value='.$value.'&'.$token.'=1');
            return '<a href="'.$url.'"><img src="'.JURI::base().$img.'" /></a>';
        }
    }
}
